@@ -1,7 +1,7 @@
 #'
 #'    rthomas.R
 #'
-#'   $Revision: 1.7 $ $Date: 2024/06/09 00:22:59 $
+#'   $Revision: 1.15 $ $Date: 2025/05/20 08:21:20 $
 #' 
 #'   Simulation of modified Thomas cluster process
 #'   using either naive algorithm or BKBC algorithm
@@ -130,12 +130,13 @@ rThomas <- local({
     function(kappa, scale, mu, win = square(1),
              nsim=1, drop=TRUE,
              ...,
+             n.cond=NULL, w.cond=NULL,
              algorithm=c("BKBC", "naive"),
              nonempty=TRUE, 
              poisthresh=1e-6,
              expand = 4*scale,
              saveparents=FALSE, saveLambda=FALSE,
-             kappamax=NULL, mumax=NULL, sigma) {
+             kappamax=NULL, mumax=NULL, LambdaOnly=FALSE, sigma) {
       ## modified Thomas process
       ## Poisson(mu) number of offspring
       ## at isotropic Normal(0,sigma^2) displacements from parent
@@ -152,6 +153,38 @@ rThomas <- local({
       check.1.real(scale)
       stopifnot(scale > 0)
 
+      ## algorithm choices
+      doLambda <- isTRUE(saveLambda) || isTRUE(LambdaOnly)
+      conditioning <- !is.null(n.cond)
+      if((conditioning || doLambda) && !isTRUE(spatstat.options("developer"))) {
+        ## The naive algorithm must be used
+        ## Change defaults
+        algorithm <- if(missing(algorithm)) "naive" else match.arg(algorithm)
+        nonempty  <- if(missing(nonempty)) FALSE else isTRUE(nonempty)
+        ## Override given arguments with a warning
+        reason <- if(conditioning) "for conditional simulation" else "for intensity calculation"
+        algorithm <- warn.reset.arg(algorithm, "naive", reason)
+        nonempty  <- warn.reset.arg(nonempty,  FALSE,   reason)
+      } else {
+        ## Any choice of algorithm is permitted
+        algorithm <- match.arg(algorithm)
+        nonempty <- isTRUE(nonempty)
+      }
+      
+      ## conditional simulation
+      if(conditioning) {
+        mod <- clusterprocess("Thomas", mu=mu, kappa=kappa, scale=scale)
+        result <- CondSimCox(mod, nsim=nsim, ...,
+                             nonempty=nonempty, algorithm=algorithm,
+                             win=win, n.cond=n.cond, w.cond=w.cond,
+                             saveparents=saveparents,
+                             saveLambda=saveLambda, LambdaOnly=LambdaOnly,
+                             drop=drop)
+        return(result)
+      }
+
+      ## ------- Unconditional simulation ------------------
+
       ## determine the effective maximum radius of clusters
       ## (for the naive algorithm, or when kappa is not constant)
       if(missing(expand))
@@ -166,21 +199,13 @@ rThomas <- local({
 
       ## detect trivial case where patterns are empty
       if(kappamax == 0 || mumax == 0) {
-        empt <- ppp(window=win)
-        if(saveparents) {
-          attr(empt, "parents") <- list(x=numeric(0), y=numeric(0))
-          attr(empt, "parentid") <- integer(0)
-          attr(empt, "cost") <- 0
-        }
-        if(saveLambda) 
-          attr(empt, "Lambda") <- as.im(0, W=win)
-        result <- rep(list(empt), nsim)
+        result <- emptyNeyScot(win, nsim,
+                               saveLambda, saveparents, LambdaOnly, ...)
         return(simulationresult(result, nsim=nsim, drop=drop))
       }
 
       #' determine algorithm
-      algorithm <- match.arg(algorithm)
-      do.parents <- saveparents || saveLambda || !is.numeric(kappa)
+      do.parents <- saveparents || doLambda || !is.numeric(kappa)
       do.hybrid <- (algorithm == "BKBC") && nonempty 
 
       if(do.hybrid) {
@@ -206,7 +231,8 @@ rThomas <- local({
           if(is.function(mu)) mu <- as.im(mu, W=win, ...)
           kapmu <- kappa * mu
           result <- rpoispp(kapmu, win=win, nsim=nsim, drop=drop, warnwin=FALSE)
-          result <- fakeNeyScot(result, kapmu, win, saveLambda, saveparents)
+          result <- fakeNeyScot(result, kapmu, win,
+                                saveLambda, saveparents, LambdaOnly)
           return(result)
         }
 
@@ -221,14 +247,22 @@ rThomas <- local({
                                kappamax=kappamax, mumax=mumax)
       }
       
-      if(saveLambda){
+      if(doLambda){
         BW <- Frame(win)
         for(i in 1:nsim) {
           parents <- attr(result[[i]], "parents")
           BX <- boundingbox(BW, bounding.box.xy(parents))
           parents <- as.ppp(parents, W=BX, check=FALSE)
           Lambda <- clusterfield("Thomas", parents, scale=scale, mu=mu, ...)
-          attr(result[[i]], "Lambda") <- Lambda[win, drop=FALSE]
+          Lambda <- Lambda[win, drop=FALSE]
+          if(LambdaOnly) {
+            #' save only the intensity
+            result[[i]] <- Lambda
+            if(saveparents) attr(result[[i]], "parents") <- parents
+          } else {
+            #' usual case - save intensity as attribute
+            attr(result[[i]], "Lambda") <- Lambda
+          }
         }
       }
       return(simulationresult(result, nsim, drop))
@@ -237,3 +271,4 @@ rThomas <- local({
   rThomas
 })
 
+
